#ifndef gf_included_gamegrid_h
#define gf_included_gamegrid_h

#include "piece.h"
#include "raceq.h"

// You might wonder why I don't use the two values below at the moment.  I'm not really too sure myself, I just didn't want
// to continue calling those constants, even from other c files.  Seemed kind of wrong to me, even though I know it works.

#define STARTGRIDHEIGHT	21
#define STARTGRIDWIDTH	10

/*
Gamegrids keep track of a lot of info.  First, there's the actual gaming grid.  Here we have three layers of data.
The first layer is for the piece that is currently in motion.
The second layer is for pieces that have stopped moving.
The third layer is the colour value for the stopped pieces and has no relation to the first layer.

The reason I separated the moving piece from the stopped ones, is because I didn't want to account for reactions between
empty spaces in the moving piece array and the blocks that have already stopped.  I tried doing it this way the first time,
and it ended up being too much work.  So, a little more space is being used to save myself and the computer a lot of time.

The colour mapping is used for pieces that are still.  The "colour" of the moving piece is simply the colour being used at 
the time.
===
Also, the gamegrid must keep track of certain options.  For example, how many lines must the player clear before he moves
to the next level?  What is his starting and (eventually) ending max level?  What is the speed of that level?  What is
his current score?  How many times should he be bumped up due to a penalty (most likely from the other player)?
What is the current piece that's moving?  What's the next piece that's due to arrive on the screen shortly?
*/

struct gamegrid
{
 int BlockArray[STARTGRIDHEIGHT][STARTGRIDWIDTH][3];	//[0] means moving array. [1] is set in stone. [2] is colour mapping.
 int GridHeight;	//Size of the playing field.
 int GridWidth;

 int LinesClear;	//How many lines cleared so far.
 int LinesRequired;	//How many lines required to clear a level.
 int CurrentLevel;	//Obvious.
 int MaxLevel;		//The currentlevel is never higher than this.
 int Speed;		//This speed is measured in milliseconds * 10.

 int Score;		//Score is determined by a number of factors.
 int ScoreCounter;	//How many lines that was cleared by the last score increase.
 int BumpMeUp;		//How many times the playing field should be bumped up due to penalty.
 int Multiplier;	//The multiplier for the score depends on the level.

 int Lose;		//If the player has lost or not.
 int GoDown;		//This signifies to the driver that the currentpiece has stopped.  Penalties and other issues are determined from here.

 int CurrentColour;	//The current colour of the piece.  You COULD just get it from CurrentPiece, except remember that Level
 Piece CurrentPiece;	// is also a factor for the colour of the pieces.
 Piece NextPiece;	//The next piece that will be dropped.
 RaceQ PieceQ;		//A list of random values that will determine nextPiece.
 int NeedNewPiece;	//Warns the driver that the gamegrid needs a new piece for next piece before it can continue.
};

typedef struct gamegrid GameGrid;

//Deletes the current row and moves everything above it down.
GameGrid ClearLine(GameGrid theGrid, int row);
//Checks all the lines surrounding the piece that just dropped to see if the player cleared a line.
GameGrid CheckLines(GameGrid theGrid);
//Stops the piece that's moving and puts it in the stopped layer of the playing field.
GameGrid StopPiece(GameGrid theGrid);
//Sets up the playing field to be played using options given.
GameGrid InitializeGrid(int StartLevel, int MaxLevel, int LinesRequired);
//Bump up the playing field by one using 7 randomnly positioned blocks.
GameGrid BumpUp(GameGrid theGrid);
//Takes the next piece and puts it on the playing field.
GameGrid NextPiece(GameGrid theGrid);
//Moves the piece down.
GameGrid MoveDown(GameGrid theGrid);
//left.
GameGrid MoveLeft(GameGrid theGrid);
//right.
GameGrid MoveRight(GameGrid theGrid);
//Some rotating piece action.
GameGrid Clockwise(GameGrid theGrid);
GameGrid Counter_Clockwise(GameGrid theGrid);

#endif
